@@ -1,18 +1,12 @@
 import os
 import subprocess
 import sys
+from itertools import chain
 
 import click
 
-GCC_ARGS = [
-    'gcc',
-    '-std=gnu11',
-    '-g',
-    '-Werror',
-    '-Wall',
-    '-Wextra',
-    '-ftrapv',                    # catch signed overflow on addition, subtraction, multiplication operations
-]
+from kks.util.config import find_target
+
 
 GPP_ARGS = [
     'g++',
@@ -35,26 +29,31 @@ ASAN_ENV = {
     'ASAN_OPTIONS': 'color=always',
 }
 
-LINK_ARGS = [
-    '-lm',
-]
-
 VALGRIND_ARGS = [
     'valgrind',
     '--leak-check=full',
 ]
 
 
-def compile_solution(directory, options):
-    c_files = list(directory.glob('*.c'))
+def compile_solution(directory, target_name, verbose, options):
+    target = find_target(target_name)
+    if target is None:
+        click.secho(f'No target {target_name} found', fg='red', err=True)
+        return None
 
-    if len(c_files) == 0:
-        click.secho('No .c files found', fg='yellow', err=True)
+    if verbose:
+        click.secho(f'Selected target: {target}')
+
+    # gcc can compile c and asm files together, so everything should be ok
+    source_files = list(chain(*[directory.glob(f) for f in target.files]))
+
+    if len(source_files) == 0:
+        click.secho('No source files found', fg='yellow', err=True)
         return None
 
     click.secho('Compiling... ', fg='green', err=True, nl=False)
 
-    binary = compile_c(directory, c_files, options)
+    binary = compile_c(directory, source_files, target, verbose, options)
 
     if binary is None:
         click.secho('Compilation failed!', fg='red', err=True)
@@ -66,29 +65,45 @@ def compile_solution(directory, options):
     return binary
 
 
-def compile_c(workdir, files, options):
-    return compile_gnu(workdir, files, options, list(GCC_ARGS))
+def compile_c(workdir, files, target, verbose, options):
+    compiler_args = ['gcc'] + target.flags
+    if not target.asm64bit and any(f.suffix.lower() == '.s' for f in files):
+        compiler_args.append('-m32')
+    return compile_gnu(
+            workdir,
+            files,
+            options,
+            compiler_args,
+            linker_args=[f'-l{lib}' for lib in target.libs],
+            out_file=target.out,
+            verbose=verbose
+    )
 
 
 def compile_cpp(workdir, files, options):
-    return compile_gnu(workdir, files, options, list(GPP_ARGS))
+    return compile_gnu(workdir, files, options, GPP_ARGS)
 
 
-def compile_gnu(workdir, files, options, compiler_args):
+def compile_gnu(workdir, files, options, compiler_args, linker_args=[], out_file='', verbose=False):
     filenames = [path.absolute() for path in files]
 
     command = compiler_args
     if options.asan:
         command += ASAN_ARGS
+    if out_file:
+        command += ['-o', (workdir / out_file).absolute()]
     command += filenames
-    command += LINK_ARGS
+    command += linker_args
+
+    if verbose:
+        click.secho('\nExecuting "{}"'.format(' '.join(map(str, command))))
 
     p = subprocess.run(command, cwd=workdir)
 
     if p.returncode != 0:
         return None
 
-    return workdir / 'a.out'
+    return workdir / (out_file or 'a.out')
 
 
 def run_solution(binary, args, options, test_data, capture_output=True):
